@@ -10,17 +10,13 @@
 #
 ###############################################################################
 
-# Find the relative path from pwd to the directory holding this file
-includes=${BASH_SOURCE[0]}
-includes=${includes%/*}/
-
 echo === $OUTPUTFILE ===
 
 endian=`file -L /proc/$$/exe`
-if expr "$endian" : '.* MSB executable.*' >&/dev/null
+if expr "$endian" : '.* MSB \+\(executable\|shared object).*' >&/dev/null
 then
     endian=BE
-elif expr "$endian" : '.* LSB executable.*' >&/dev/null
+elif expr "$endian" : '.* LSB \+\(executable\|shared object\).*' >&/dev/null
 then
     endian=LE
 else
@@ -33,9 +29,21 @@ maxtypelen=31
 maxtype=`for ((i=0; i<$((maxtypelen)); i++)); do echo -n a; done`
 
 PAGE_SIZE=`getconf PAGESIZE`
-maxdesclen=$((PAGE_SIZE - 1))
-maxdesc=`for ((i=0; i<$((maxdesclen)); i++)); do echo -n a; done`
-maxcall=$maxdesc
+pagelen=$((PAGE_SIZE - 1))
+fullpage=`for ((i=0; i<$((pagelen)); i++)); do echo -n a; done`
+string4095=`for ((i=0; i<4095; i++)); do echo -n a; done`
+
+if kernel_at_or_later_than 3.18
+then
+    maxdesc=$string4095
+elif rhel6_kernel_at_or_later_than 2.6.32-589.el6
+then
+    maxdesc=$string4095
+else
+    maxdesc=$fullpage
+fi
+
+maxcall=$fullpage
 
 maxsquota=`grep '^ *0': /proc/key-users | sed s@.*/@@`
 
@@ -76,58 +84,35 @@ function toolbox_report_result()
 {
     if [ $RUNNING_UNDER_RHTS = 1 ]
     then
-	report_result $TEST $result
+	report_result $1 $2
     fi
-    if [ $result = FAIL ]
+    if [ $2 = FAIL ]
     then
 	exit 1
     fi
 }
 
-. $includes/version.inc.sh
-
-###############################################################################
-#
-# Return true if the keyutils package being tested is older than the given
-# version.
-#
-###############################################################################
-function keyutils_older_than ()
+function toolbox_skip_test()
 {
-    version_less_than $KEYUTILSVER $1
+    echo "++++ SKIPPING TEST" >>$OUTPUTFILE
+    marker "$2"
+    toolbox_report_result $1 PASS
 }
 
 ###############################################################################
 #
-# Return true if the keyutils package being tested is at or later than the
-# given version.
+# Return true if the command is found in $PATH. If not, log that the test is
+# being skipped, report the result as PASS, and exit.
 #
 ###############################################################################
-function keyutils_at_or_later_than ()
+function require_command ()
 {
-    ! keyutils_older_than $1
-}
-
-###############################################################################
-#
-# Return true if the keyutils package being tested is newer than the given
-# version.
-#
-###############################################################################
-function keyutils_newer_than ()
-{
-    version_less_than $1 $KEYUTILSVER
-}
-
-###############################################################################
-#
-# Return true if the keyutils package being tested is at or older than the
-# given version.
-#
-###############################################################################
-function keyutils_at_or_older_than ()
-{
-    ! keyutils_newer_than $1
+    which "$1" >&/dev/null
+    if [ $? != 0 ]
+    then
+	toolbox_skip_test "SKIP DUE TO MISSING COMMAND: $1"
+        exit 0
+    fi
 }
 
 ###############################################################################
@@ -685,6 +670,25 @@ function expect_payload ()
     fi
 }
 
+###############################################################################
+#
+# extract multiline output from the log file
+#
+###############################################################################
+function expect_multiline ()
+{
+    my_varname=$1
+    my_linecount="`echo \"$2\" | wc -l`"
+
+    my_payload=$(tail -$my_linecount $OUTPUTFILE)
+    eval $my_varname="\"$my_payload\""
+
+    if [ $# != 2 -o "x$my_payload" != "x$2" ]
+    then
+	failed
+    fi
+}
+
 ###############################################################################
 #
 # revoke a key
@@ -1078,6 +1082,28 @@ function invalidate_key ()
     fi
 }
 
+###############################################################################
+#
+# Do a DH computation
+#
+###############################################################################
+function dh_compute ()
+{
+    my_exitval=0
+    if [ "x$1" = "x--fail" ]
+    then
+	my_exitval=1
+	shift
+    fi
+
+    echo keyctl dh_compute $@ >>$OUTPUTFILE
+    keyctl dh_compute $@ >>$OUTPUTFILE 2>&1
+    if [ $? != $my_exitval ]
+    then
+	failed
+    fi
+}
+
 ###############################################################################
 #
 # Make sure we sleep at least N seconds
@@ -1108,4 +1134,3 @@ function set_gc_delay()
         echo "Set $key_gc_delay_file to $delay, orig: $orig_gc_delay"
     fi
 }
-
